@@ -529,7 +529,7 @@ z_stream_ctrl_method(ZStream *s, guint function, gpointer value, guint vlen)
           res = TRUE;
         }
       break;
-      
+
     default:
       if (s->child)
         {
@@ -604,6 +604,44 @@ z_stream_set_cond(ZStream *s, guint type, gboolean value)
   return ret;
 }
 
+/**
+ * This function returns the value of the appropriate want_read/want_write flags.
+ *
+ * @param[in] s ZStream instance
+ * @param[out] v store actual value here; not modified if returns FALSE; NULL-tolerant
+ *
+ * @returns TRUE on success
+ **/
+gboolean
+z_stream_get_cond(ZStream *s, guint type, gboolean *v)
+{
+  gboolean ret = FALSE;
+  gboolean value;
+
+  switch (type)
+    {
+    case G_IO_IN:
+      ret = z_stream_ctrl(s, ZST_CTRL_SET_COND_READ, &value, sizeof(value));
+      break;
+
+    case G_IO_OUT:
+      ret = z_stream_ctrl(s, ZST_CTRL_SET_COND_WRITE, &value, sizeof(value));
+      break;
+
+    case G_IO_PRI:
+      ret = z_stream_ctrl(s, ZST_CTRL_SET_COND_PRI, &value, sizeof(value));
+      break;
+
+    default:
+      break;
+    }
+
+  if (ret && v)
+    *v = value;
+
+  return ret;
+}
+
 /**
  * This function attaches an I/O callback function to stream.
  *
@@ -793,6 +831,7 @@ z_stream_save_context(ZStream *self, ZStreamContext *context)
   context->write_cb = self->write_cb;
   
   context->timeout = self->timeout;
+  context->nonblocking = z_stream_get_nonblock(self);
 
   self->want_read = FALSE;
   self->user_data_read = NULL;
@@ -851,6 +890,7 @@ z_stream_restore_context(ZStream *self, ZStreamContext *context)
   self->write_cb = context->write_cb;
   
   self->timeout = context->timeout;
+  z_stream_set_nonblock(self, context->nonblocking);
   
   if (context->stream_extra)
     {
@@ -1202,11 +1242,15 @@ z_stream_read_chunk(ZStream *self, void *buf, gsize len, gsize *bytes_read, GErr
   GIOStatus status = G_IO_STATUS_NORMAL;
   gsize bytes;
 
+  /* Only works on a blocking stream. */
+  g_return_val_if_fail(z_stream_get_nonblock(self) == FALSE, G_IO_STATUS_ERROR);
+
   g_return_val_if_fail((error == NULL) || (*error == NULL), G_IO_STATUS_ERROR);
 
   z_enter();
   *bytes_read = 0;
-  while (status == G_IO_STATUS_NORMAL && len > 0)
+  while (len > 0 &&
+         (status == G_IO_STATUS_NORMAL || status == G_IO_STATUS_AGAIN))
     {
       status = z_stream_read(self, buf, len, &bytes, error);
       if (status == G_IO_STATUS_NORMAL)
@@ -1295,7 +1339,10 @@ z_stream_unget_packet_method(ZStream *self, ZPktBuf *pack, GError **error)
   for (p = self; p; p = p->child)
     {
       if ((p->umbrella_flags & G_IO_IN))
-        p->ungot_bufs = g_list_prepend(p->ungot_bufs, pack);
+        {
+          p->ungot_bufs = g_list_prepend(p->ungot_bufs, pack);
+          break;
+        }
     }
   z_return(TRUE);
 }
@@ -1471,11 +1518,4 @@ ZStreamFuncs z_stream_funcs =
 #ifdef G_OS_WIN32
   LIBZORPLL_EXTERN
 #endif
-ZClass ZStream__class = 
-{
-  Z_CLASS_HEADER,
-  &ZObject__class,
-  "ZStream",
-  sizeof(ZStream),
-  &z_stream_funcs.super,
-};
+Z_CLASS_DEF(ZStream, ZObject, z_stream_funcs);
