@@ -488,10 +488,10 @@ z_stream_fd_write_method(ZStream *stream,
  **/
 GIOStatus
 z_stream_fd_write_pri_method(ZStream *stream,
-		             const gchar *buf,
-		             gsize count,
-		             gsize *bytes_written,
-		             GError **error)
+                             const void *buf,
+                             gsize count,
+                             gsize *bytes_written,
+                             GError **error)
 {
   ZStreamFD *self = (ZStreamFD *) stream;
   int res, attempt = 1;
@@ -539,6 +539,7 @@ z_stream_fd_write_pri_method(ZStream *stream,
   g_clear_error(error);
   g_set_error (error, G_IO_CHANNEL_ERROR,
                g_io_channel_error_from_errno (z_errno_get()),
+               "%s",
                strerror (z_errno_get()));
   z_return(G_IO_STATUS_ERROR);
 }
@@ -594,8 +595,9 @@ z_stream_fd_shutdown_method(ZStream *stream, int i, GError **error)
   if (res != 0)
     {
       g_set_error (error, G_IO_CHANNEL_ERROR,
-                          g_io_channel_error_from_errno (z_errno_get()),
-                          strerror (z_errno_get()));
+                   g_io_channel_error_from_errno (z_errno_get()),
+                   "%s",
+                   strerror (z_errno_get()));
       z_return(G_IO_STATUS_ERROR);
     }
   z_return(G_IO_STATUS_NORMAL);
@@ -872,7 +874,7 @@ z_stream_fd_extra_restore_method(ZStream *s, gpointer extra)
  * @returns ZStream instance
  **/
 ZStream *
-z_stream_fd_new(gint fd, gchar *name)
+z_stream_fd_new(gint fd, const gchar *name)
 {
   ZStreamFD *self;
 
@@ -944,11 +946,4 @@ z_stream_fd_funcs =
 /**
  * ZStreamFD class descriptor.
  **/
-ZClass ZStreamFD__class =
-{
-  Z_CLASS_HEADER,
-  &ZStream__class,
-  "ZStreamFD",
-  sizeof(ZStreamFD),
-  &z_stream_fd_funcs.super,
-};
+Z_CLASS_DEF(ZStreamFD, ZStream, z_stream_fd_funcs);
