@@ -658,6 +658,20 @@ z_stream_gzip_ctrl_method(ZStream *stream, guint function, gpointer value, guint
     case ZST_CTRL_SET_CALLBACK_PRI:
       ret = z_stream_ctrl_method(stream, function, value, vlen);
       break;
+
+    case ZST_CTRL_GET_BUFFERED_BYTES:
+      if (vlen == sizeof(gsize))
+        {
+          gsize *size = (gsize *) value;
+          ZStreamGzip *self = Z_CAST(stream, ZStreamGzip);
+
+          *size += self->decode.avail_in;
+
+          ret = TRUE;
+          if (stream->child)
+            ret = z_stream_ctrl(stream->child, ZST_CTRL_MSG_FORWARD | function, value, vlen);
+        }
+      break;
       
     default:
       ret = z_stream_ctrl_method(stream, ZST_CTRL_MSG_FORWARD | function, value, vlen);
@@ -766,6 +780,8 @@ static void
 z_stream_gzip_set_child(ZStream *s, ZStream *new_child)
 {
   ZStreamGzip *self = Z_CAST(s, ZStreamGzip);
+
+  z_stream_ref(s);
   
   Z_SUPER(s, ZStream)->set_child(s, new_child);
   if (new_child)
@@ -775,6 +791,8 @@ z_stream_gzip_set_child(ZStream *s, ZStream *new_child)
 
       g_assert((self->flags & Z_SGZ_GZIP_HEADER) == 0 || z_stream_get_nonblock(new_child) == FALSE);
     }
+
+  z_stream_unref(s);
 }
 
 /**
@@ -955,11 +973,4 @@ ZStreamFuncs z_stream_gzip_funcs =
   NULL,
 };
 
-ZClass ZStreamGzip__class = 
-{
-  Z_CLASS_HEADER,
-  &ZStream__class,
-  "ZStreamGzip",
-  sizeof(ZStreamGzip),
-  &z_stream_gzip_funcs.super
-};
+Z_CLASS_DEF(ZStreamGzip, ZStream, z_stream_gzip_funcs);
