@@ -17,17 +17,24 @@
 extern "C" {
 #endif
 
-#define Z_RANDOM_STRONG 0
-#define Z_RANDOM_BASIC  1
-#define Z_RANDOM_WEAK   2
+/**
+ * Strength of random number generation algorithm.
+ **/
+typedef enum
+{
+  Z_RANDOM_STRONG=0,
+  Z_RANDOM_BASIC,
+  Z_RANDOM_WEAK,
+  Z_RANDOM_NUM_STRENGTHS
+} ZRandomStrength;
 
-gboolean z_random_sequence_get_bounded(guchar *target,
-                                        guint  len,
-                                       guchar  min,
-                                       guchar  max,
-                                        guint  type);
+gboolean
+z_random_sequence_get(ZRandomStrength strength, guchar *target, gsize target_len);
 
-#define z_random_sequence_get(target, len, type) z_random_sequence_get_bounded(target, len, 0, 255, type)
+gboolean 
+z_random_sequence_get_bounded(ZRandomStrength strength, 
+                              guchar *target, gsize target_len,
+                              guchar min, guchar max);
 
 #ifdef __cplusplus
 }
